Class: Frodo::Middleware::Authentication

Inherits:
Frodo::Middleware show all
Defined in:
lib/frodo/middleware/authentication.rb

Overview

Faraday middleware that allows for on the fly authentication of requests. When a request fails (a status of 401 is returned), the middleware will attempt to either reauthenticate (username and password) or refresh the oauth access token (if a refresh token is present).

Direct Known Subclasses

Token

Defined Under Namespace

Classes: Token

Instance Method Summary collapse

Methods inherited from Frodo::Middleware

#client, #initialize

Constructor Details

This class inherits a constructor from Frodo::Middleware

Instance Method Details

#authenticate!Object

Internal: Performs the authentication and returns the response body.



21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
# File 'lib/frodo/middleware/authentication.rb', line 21

def authenticate!
  response = connection.post '/common/oauth2/token' do |req|
    req.body = encode_www_form(params)
  end

  if response.status >= 500
    raise Frodo::ServerError, error_message(response)
  elsif response.status != 200
    raise Frodo::AuthenticationError, error_message(response)
  end

  @options[:oauth_token]  = response.body['access_token']
  @options[:refresh_token]  = response.body['refresh_token']
  @options[:authentication_callback]&.call(response.body)

  response.body
end

#call(env) ⇒ Object

Rescue from 401’s, authenticate then raise the error again so the client can reissue the request.



13
14
15
16
17
18
# File 'lib/frodo/middleware/authentication.rb', line 13

def call(env)
  @app.call(env)
rescue Frodo::UnauthorizedError
  authenticate!
  raise
end

#connectionObject

Internal: Faraday connection to use when sending an authentication request.



45
46
47
48
49
50
51
52
53
54
55
56
57
58
# File 'lib/frodo/middleware/authentication.rb', line 45

def connection
  @connection ||= Faraday.new(faraday_options) do |builder|
    builder.use Faraday::Request::UrlEncoded
    builder.response :json

    if Frodo.log?
      builder.use Frodo::Middleware::Logger,
                  Frodo.configuration.logger,
                  @options
    end

    builder.adapter @options[:adapter]
  end
end

#encode_www_form(params) ⇒ Object

Featured detect form encoding. URI in 1.8 does not include encode_www_form



67
68
69
70
71
72
73
74
75
76
77
# File 'lib/frodo/middleware/authentication.rb', line 67

def encode_www_form(params)
  if URI.respond_to?(:encode_www_form)
    URI.encode_www_form(params)
  else
    params.map do |k, v|
      k = CGI.escape(k.to_s)
      v = CGI.escape(v.to_s)
      "#{k}=#{v}"
    end.join('&')
  end
end

#error_message(response) ⇒ Object

Internal: The parsed error response.



61
62
63
# File 'lib/frodo/middleware/authentication.rb', line 61

def error_message(response)
  "#{response.body['error']}: #{response.body['error_description']}"
end

#paramsObject

Internal: The params to post to the OAuth service.

Raises:

  • (NotImplementedError)


40
41
42
# File 'lib/frodo/middleware/authentication.rb', line 40

def params
  raise NotImplementedError
end